From 817e695e32f78332d43ace023bbc82cc851fcba5 Mon Sep 17 00:00:00 2001
From: Aaron Patterson <tenderlove@ruby-lang.org>
Date: Thu, 26 May 2022 16:18:10 -0700
Subject: [PATCH 2/3] Escape untrusted text when logging

This fixes a shell escape issue

[CVE-2022-30123]
---
 lib/rack/common_logger.rb  |  3 +++
 lib/rack/lint.rb           |  2 +-
 test/spec_common_logger.rb | 12 ++++++++++++
 test/spec_lint.rb          |  5 +++++
 4 files changed, 21 insertions(+), 1 deletion(-)

diff --git a/lib/rack/common_logger.rb b/lib/rack/common_logger.rb
index ae410430..cf14fd46 100644
--- a/lib/rack/common_logger.rb
+++ b/lib/rack/common_logger.rb
@@ -54,7 +54,10 @@ module Rack
         length,
         now - began_at ]
 
+      msg.gsub!(/[^[:print:]\n]/) { |c| "\\x#{c.ord}" }
+
       logger = @logger || env[RACK_ERRORS]
+
       # Standard library logger doesn't support write but it supports << which actually
       # calls to write on the log device without formatting
       if logger.respond_to?(:write)
diff --git a/lib/rack/lint.rb b/lib/rack/lint.rb
index 683ba684..530d08a2 100644
--- a/lib/rack/lint.rb
+++ b/lib/rack/lint.rb
@@ -295,7 +295,7 @@ module Rack
       check_hijack env
 
       ## * The <tt>REQUEST_METHOD</tt> must be a valid token.
-      assert("REQUEST_METHOD unknown: #{env[REQUEST_METHOD]}") {
+      assert("REQUEST_METHOD unknown: #{env[REQUEST_METHOD].dump}") {
         env[REQUEST_METHOD] =~ /\A[0-9A-Za-z!\#$%&'*+.^_`|~-]+\z/
       }
 
diff --git a/test/spec_common_logger.rb b/test/spec_common_logger.rb
index 3589576b..7d5adcf1 100644
--- a/test/spec_common_logger.rb
+++ b/test/spec_common_logger.rb
@@ -21,6 +21,10 @@ describe Rack::CommonLogger do
     [200,
      {"Content-Type" => "text/html", "Content-Length" => "0"},
      []]}
+  app_without_lint = lambda { |env|
+    [200,
+     { "content-type" => "text/html", "content-length" => length.to_s },
+     [obj]]}
 
   it "log to rack.errors by default" do
     res = Rack::MockRequest.new(Rack::CommonLogger.new(app)).get("/")
@@ -85,6 +89,14 @@ describe Rack::CommonLogger do
     (0..1).must_include duration.to_f
   end
 
+  it "escapes non printable characters except newline" do
+    logdev = StringIO.new
+    log = Logger.new(logdev)
+    Rack::MockRequest.new(Rack::CommonLogger.new(app_without_lint, log)).request("GET\b", "/hello")
+
+    logdev.string.must_match(/GET\\x8 \/hello/)
+  end
+
   def length
     123
   end
diff --git a/test/spec_lint.rb b/test/spec_lint.rb
index d99c1aa3..202597b8 100644
--- a/test/spec_lint.rb
+++ b/test/spec_lint.rb
@@ -96,6 +96,11 @@ describe Rack::Lint do
     }.must_raise(Rack::Lint::LintError).
       message.must_match(/REQUEST_METHOD/)
 
+    lambda {
+      Rack::Lint.new(nil).call(env("REQUEST_METHOD" => "OOPS?\b!"))
+    }.must_raise(Rack::Lint::LintError).
+      message.must_match(/OOPS\?\\/)
+
     lambda {
       Rack::Lint.new(nil).call(env("SCRIPT_NAME" => "howdy"))
     }.must_raise(Rack::Lint::LintError).
-- 
2.36.0

